@@ -217,8 +217,19 @@ pop.aggregate.regional <- function(pop.pred, regions, name,
 	return(res)
 }
 
+split.pop05 <- function(dat) {
+    # split first pop age group into 0-1 & 1-4
+    res <- abind(array(NA, dim(dat)[2:3]), dat, along = 1) # add row for the first age group (0-1)
+    # using the sprague formula for P_0
+    res[1,,] <- apply(dat[1,,, drop = FALSE]*0.3616 + dat[2,,, drop = FALSE]*-0.2768 + dat[3,,, drop = FALSE]*0.1488 + dat[4,,, drop = FALSE]*-0.0336, c(2,3), sum)
+    res[2,,] <- dat[1,,] - res[1,,]
+    return(res)
+}
 
-pop.aggregate.countries <- function(pop.pred, regions, name, verbose=verbose, adjust=FALSE) {
+pop.aggregate.countries <- function(pop.pred, regions, name, 
+                                    use.kannisto = TRUE, keep.vital.events = NULL,
+                                    verbose=verbose, adjust=FALSE) {
+    
 	if(verbose) cat('\nAggregating using countries as inputs.')
 	nreg <- length(regions)
 	quantiles.to.keep <- as.numeric(dimnames(pop.pred$quantiles)[[2]])
@@ -227,7 +238,11 @@ pop.aggregate.countries <- function(pop.pred, regions, name, verbose=verbose, ad
 						dimnames=c(list(regions), dimnames(pop.pred$quantilesMage)[2:4]))
 	mean_sd <- mean_sdM <- mean_sdF <- array(NA, c(nreg,dim(pop.pred$traj.mean.sd)[2:3]))
 	obs.data <- pop.pred$inputs$pop.matrix
-	aggr.obs.data <- list(male=NULL, female=NULL)
+	# convert to arrays for performance reasons
+	for(sex in names(obs.data)) 
+	    obs.data[[sex]] <- data.matrix(obs.data[[sex]])
+
+	aggr.obs.data <- aggrobs <- abrpop <- abrpopobs <- aggrabrpop <- aggrabrpopobs <- abrd <- aggrabrdeaths <- abrdobs <- aggrabrdeathsobs <- abrpop.hch <- aggrabrpop.hch <- abrd.hch <- aggrabrdeaths.hch <- list(male=NULL, female=NULL)
 	outdir <- gsub('predictions', paste('aggregations', name, sep='_'), pop.output.directory(pop.pred))
 	if(file.exists(outdir)) unlink(outdir, recursive=TRUE)
 	dir.create(outdir, recursive=TRUE)
@@ -236,15 +251,22 @@ pop.aggregate.countries <- function(pop.pred, regions, name, verbose=verbose, ad
 	valid.regions <- rep(FALSE, length(regions))
 	status.for.gui <- paste('out of', nreg, 'regions.')
 	gui.options <- list()
-	has.vital.events <- FALSE
+	no.vital.events <- !is.null(keep.vital.events) && keep.vital.events == FALSE
 	aggr.quantities <- c('totp', 'totpm', 'totpf', 'totp.hch', 'totpm.hch', 'totpf.hch')
 	aggr.quantities.ve <- c('btm', 'btf', 'btm.hch', 'btf.hch', 
 								'deathsm', 'deathsf', 'deathsm.hch', 'deathsf.hch',
 								'migm', 'migf')
+	ind.names <- list(male = list(pop = "totpm", mx = "mxm"), 
+	                 female = list(pop = "totpf", mx = "mxf"))
+	ind.names.hch <- list(male = list(pop = "totpm.hch", mx = "mxm.hch"), 
+	                  female = list(pop = "totpf.hch", mx = "mxf.hch"))
 	# The next two lines need to be there for the R checker to get to know these objects
 	totp <- totpm <- totpf <- totp.hch <- totpm.hch <- totpf.hch <- NULL
 	btm <- btf <- deathsm <- deathsf <- migm <- migf <- btm.hch <- btf.hch <- deathsm.hch <- deathsf.hch <- NULL
 	aggr.quantities.all <- aggr.quantities
+	obs.cols.ve <- paste(as.integer(colnames(obs.data[["male"]]))-5, 
+	                     colnames(obs.data[["male"]]), sep = "-")
+	
 	for(reg.idx in 1:length(regions)) {
 		if(getOption('bDem.PopAgpred', default=FALSE)) {
 			# This is to unblock the GUI, if the run is invoked from bayesDem
@@ -262,59 +284,133 @@ pop.aggregate.countries <- function(pop.pred, regions, name, verbose=verbose, ad
 		countries.index <- which(is.element(pop.pred$countries[,'code'], countries))
 		e <- new.env()
 		if(adjust && is.null(pop.pred$adjust.env)) pop.pred$adjust.env <- new.env()
-		
+		prev.year <- as.character(pop.pred$proj.years.pop[1]-5)
+		#debug.df <- NULL
 		for(cidx in 1:length(countries.index)) {
 			country.obs.idx <- grep(paste('^', countries[cidx], '_', sep=''), rownames(obs.data[['male']]), value=FALSE)
 			traj.file <- file.path(pop.output.directory(pop.pred), paste('totpop_country', countries[cidx], '.rda', sep=''))
 			load(traj.file, envir=e)
 			if(adjust) adjust.trajectories(countries[cidx], e, pop.pred, pop.pred$adjust.env)
-			if(has.vital.events || cidx == 1) {
+			if(!no.vital.events) {
 				ve.file <- file.path(pop.output.directory(pop.pred), 
 									paste('vital_events_country', countries[cidx], '.rda', sep=''))
 				if(file.exists(ve.file)) {
 					if(cidx == 1) {
-						has.vital.events <- TRUE
+					    #requireNamespace("DemoTools")
 						aggr.quantities.all <- c(aggr.quantities, aggr.quantities.ve)
 						observed <- list()
 					}
 					load(ve.file, envir=e)
+					use.obs.cols <- intersect(obs.cols.ve, colnames(e$observed$mxm))
+					use.obs.cols.pop <- substr(use.obs.cols, 6, 9)
+					
+					# compute abridged population and deaths
+					for(sex in c("male", "female")) {
+					    # Attach the second last observed year of population
+					    #popavg <- abind(abind(abind(c(obs.data[[sex]][country.obs.idx, prev.year], rep(0, 6)), # fill in old ages
+					    #             NULL, along = 2, new.names = prev.year), NULL, along = 3)[,,rep(1,dim(e[[ind.names[[sex]]$pop]])[3]), drop = FALSE],
+					    #           e[[ind.names[[sex]]$pop]], along = 2)
+					    popavg <- array(0, c(27, dim(e[[ind.names[[sex]]$pop]])[2]+1, dim(e[[ind.names[[sex]]$pop]])[3]))
+					    popavg[1:length(country.obs.idx),1,] <- obs.data[[sex]][country.obs.idx, prev.year]
+					    popavg[,-1,] <- e[[ind.names[[sex]]$pop]]
+					    popavg <- (popavg[,-1, ,drop = FALSE] + popavg[,-dim(popavg)[2],, drop = FALSE])/2.
+					    
+					    #popavg.hch <- abind(abind(abind(c(obs.data[[sex]][country.obs.idx, prev.year], rep(0, 6)), # fill in old ages
+					    #                            NULL, along = 2, new.names = prev.year), NULL, along = 3)[,,rep(1,2)],
+					    #                e[[paste0(ind.names[[sex]]$pop, ".hch")]], along = 2)
+					    popavg.hch <- array(0, c(27, dim(e[[ind.names[[sex]]$pop]])[2]+1, 2))
+					    popavg.hch[1:length(country.obs.idx),1,] <- obs.data[[sex]][country.obs.idx, prev.year]
+					    popavg.hch[,-1,] <- e[[ind.names.hch[[sex]]$pop]]
+					    popavg.hch <- (popavg.hch[,-1, ,drop = FALSE] + popavg.hch[,-dim(popavg)[2],, drop = FALSE])/2.
+					    # projected abridged pop and deaths
+					    abrpop[[sex]] <- split.pop05(popavg)
+					    abrpop.hch[[sex]] <- split.pop05(popavg.hch)
+					    abrd[[sex]] <- abrpop[[sex]] * e[[ind.names[[sex]]$mx]] # should be multiplied by five but that would get canceled out later
+					    abrd.hch[[sex]] <- abrpop.hch[[sex]] * e[[ind.names.hch[[sex]]$mx]]
+					    # observed abridged pop  and deaths
+					    popavgobs <- (obs.data[[sex]][country.obs.idx, -1 , drop=FALSE] + 
+					                    obs.data[[sex]][country.obs.idx, -dim(obs.data[[sex]])[2] , drop=FALSE])/2.
+					    abrpopobs[[sex]] <- split.pop05(abind(popavgobs[,use.obs.cols.pop, drop = FALSE], NULL, along = 3))
+					    abrdobs[[sex]] <- abrpopobs[[sex]]*e$observed[[ind.names[[sex]]$mx]][,use.obs.cols,, drop = FALSE]
+					}
+					# debug.df <- rbind(debug.df, data.frame(
+					#                     country = countries[cidx],
+					#                     age = c(0,1, seq(5, 130, by = 5)),
+					#                     #popMabr_2015 = c(spltmpM[,"2010-2015",], rep(NA, 6)),
+					#                   #mxM_2010_15 = c(e$observed$mxm[,"2010-2015",], rep(NA, 6)),
+					#                   #deathsM_2010_15 = c(abrdobs[["male"]][,"2010-2015"], rep(NA, 6)),
+					#                   popM_2015 = c(NA, obs.data[["male"]][country.obs.idx,"2015",], rep(NA, 6)),
+					#                   popM_2020 = c(NA, e$totpm[,"2020",]),
+					#                   popMabr_2020 = split.pop05(e$totpm)[,"2020",], 
+					#                   mxM_2015_20 = e$mxm[,"2018",],
+					#                   deathsM_2015_20 = abrd[["male"]][,"2020",],
+					#                   popM_2025 = c(NA, e$totpm[,"2025",]),
+					#                   popMabr_2025 = split.pop05(e$totpm)[,"2025",],
+					#                   mxM_2020_25 = e$mxm[,"2023",],
+					#                   deathsM_2020_25 = abrd[["male"]][,"2025",])
+					# )
+				} else {
+				    no.vital.events <- TRUE
 				}
 			}
+			#debug.dt <- data.table(debug.df)
+			#debug.dt[]
 			if(cidx == 1) {
 				for(par in aggr.quantities.all)
 					assign(par, e[[par]])
-				aggr.obs.dataM <- obs.data[['male']][country.obs.idx,, drop=FALSE]
-				aggr.obs.dataF <- obs.data[['female']][country.obs.idx,, drop=FALSE]
-				rownames(aggr.obs.dataM) <- rownames(aggr.obs.dataF) <- sub(paste(countries[cidx], '_', sep=''), 
+			    for(sex in names(aggrobs)) 
+				    aggrobs[[sex]] <- obs.data[[sex]][country.obs.idx,, drop=FALSE]
+				rownames(aggrobs[["male"]]) <- rownames(aggrobs[["female"]]) <- sub(paste(countries[cidx], '_', sep=''), 
 																paste(id, '_', sep=''), rownames(obs.data[['male']][country.obs.idx,, drop=FALSE]))
-				if(has.vital.events) {
+				if(!no.vital.events) {
 					for(par in aggr.quantities.ve)
 						if(!is.null(e$observed[[par]]))
 							observed[[par]] <- e$observed[[par]]
+					for(sex in names(aggrabrdeaths)) {
+					    aggrabrdeaths[[sex]] <- abrd[[sex]]
+					    aggrabrdeathsobs[[sex]] <- abrdobs[[sex]]
+					    aggrabrdeaths.hch[[sex]] <- abrd.hch[[sex]]
+					    aggrabrpop[[sex]] <- abrpop[[sex]]
+					    aggrabrpop.hch[[sex]] <- abrpop.hch[[sex]]
+					    aggrabrpopobs[[sex]] <- abrpopobs[[sex]]
+					}
 				}
 				trajectory.indices <- e$trajectory.indices											
 			} else {
 				for(par in aggr.quantities.all)
 					assign(par, get(par) + e[[par]])
-				aggr.obs.dataM <- aggr.obs.dataM + obs.data[['male']][country.obs.idx,]
-				aggr.obs.dataF <- aggr.obs.dataF + obs.data[['female']][country.obs.idx,]
-				if(has.vital.events) {
+			    for(sex in names(aggrobs)) 
+			        aggrobs[[sex]] <- aggrobs[[sex]] + obs.data[[sex]][country.obs.idx,]
+				if(!no.vital.events) {
 					for(par in names(observed))
 						observed[[par]] <- observed[[par]] + e$observed[[par]]
+					for(sex in names(aggrabrdeaths)) {
+					    aggrabrpop[[sex]] <- aggrabrpop[[sex]] + abrpop[[sex]]
+					    aggrabrpopobs[[sex]] <- aggrabrpopobs[[sex]] + abrpopobs[[sex]]
+					    aggrabrpop.hch[[sex]] <- aggrabrpop.hch[[sex]] + abrpop.hch[[sex]]
+					    aggrabrdeaths[[sex]] <- aggrabrdeaths[[sex]] + abrd[[sex]]
+					    aggrabrdeaths.hch[[sex]] <- aggrabrdeaths.hch[[sex]] + abrd.hch[[sex]]
+					    aggrabrdeathsobs[[sex]] <- aggrabrdeathsobs[[sex]] + abrdobs[[sex]]
+					}
 				}
 			}
 		}
 		save(totp, totpm, totpf, totp.hch, totpm.hch, totpf.hch, trajectory.indices,
 			 file = file.path(outdir, paste0('totpop_country', id, '.rda')))
-		if(has.vital.events) {
+		if(!no.vital.events) {
+		    popfwprev <- array(0, c(7, dim(totpf)[2]+1, dim(totpf)[3]))
+		    popfwprev[,1,] <- aggrobs[["female"]][4:10, prev.year]
+		    popfwprev[,-1,] <- totpf[4:10,,]
+		    #tmp <- abind(aggrobs[["female"]][4:10, prev.year, drop=FALSE], NULL, along=3)
+		    #popfwprev <- abind(tmp[,,rep(1,dim(totpf)[3]), drop=FALSE], totpf[4:10,,,drop=FALSE], along=2)
+            #stop("")
 			# asfert
-			# Attach the second last observed year of female population in order to get last observed fertility
-			tmp <- aggr.obs.dataF[4:10,as.character(pop.pred$proj.years.pop[1]-5),drop=FALSE]
-			tmp <- abind(tmp, NULL, along=3)
-			popfwprev <- abind(tmp[,,rep(1,dim(totpf)[3]), drop=FALSE], totpf[4:10,,,drop=FALSE], along=2)
 			asfert <- 2*(btm + btf)/(popfwprev[,-dim(popfwprev)[2],,drop=FALSE]+
 										popfwprev[,-1,,drop=FALSE])
-			popfwprev.hch <- abind(tmp[,,rep(1,dim(totpf.hch)[3]), drop=FALSE], totpf.hch[4:10,,,drop=FALSE], along=2)
+			popfwprev.hch <- array(0, c(7, dim(totpf)[2]+1, dim(totpf.hch)[3]))
+			popfwprev.hch[,1,] <- aggrobs[["female"]][4:10, prev.year]
+			popfwprev.hch[,-1,] <- totpf.hch[4:10,,]
+			#popfwprev.hch <- abind(tmp[,,rep(1,dim(totpf.hch)[3]), drop=FALSE], totpf.hch[4:10,,,drop=FALSE], along=2)
 			asfert.hch <- 2*(btm.hch + btf.hch)/(popfwprev.hch[,-dim(popfwprev.hch)[2],,drop=FALSE]+
 										popfwprev.hch[,-1,,drop=FALSE])
 			# pasfert
@@ -323,23 +419,47 @@ pop.aggregate.countries <- function(pop.pred, regions, name, verbose=verbose, ad
 			tfr.hch <- apply(asfert.hch, c(2,3), sum)
 			pasfert.hch <- asfert.hch/abind(tfr.hch, NULL, along=0)[rep(1,dim(asfert.hch)[1]),,,drop=FALSE]*100
 			
-			# TODO: mxm, mxf, mxm.hch, mxf.hch
-			
+			# compute mortality rates from aggregated abridged deaths and population
+			mxm <- aggrabrdeaths[["male"]] / aggrabrpop[["male"]] 
+			mxf <- aggrabrdeaths[["female"]] / aggrabrpop[["female"]]
+			mxm.hch <- aggrabrdeaths.hch[["male"]] / aggrabrpop.hch[["male"]] 
+			mxf.hch <- aggrabrdeaths.hch[["female"]] / aggrabrpop.hch[["female"]]
+			dimnames(mxm)[1:2] <- dimnames(mxf)[1:2] <- dimnames(mxm.hch)[1:2] <- dimnames(mxf.hch)[1:2] <- list(
+			    c(0,1,seq(5, length = dim(mxm)[1]-2, by = 5)), dimnames(btm)[[2]])
+
+			# apply kannisto for old age groups
+			if(use.kannisto) {
+			    for(itraj in 1:dim(mxm)[3]) {
+			        kan <- do.call("cokannisto", c(list(mxm[1:21,,itraj], mxf[1:21,,itraj])))
+			        mxm[,,itraj] <- kan$male
+			        mxf[,,itraj] <- kan$female
+			    }
+			    for(ivar in 1:dim(mxm.hch)[3]) {
+			        kan.hch <- do.call("cokannisto", c(list(mxm.hch[1:21,,ivar], mxf.hch[1:21,,ivar])))
+			        mxm.hch[,,ivar] <- kan.hch$male
+			        mxf.hch[,,ivar] <- kan.hch$female
+			    }
+			}
 			# asfert, pasfert for observed data
 			observed <- within(observed, {
-				tmp <- abind(aggr.obs.dataF[4:10, , drop=FALSE], NULL, along=3)
+			    tmp <- abind(aggrobs[["female"]][4:10, , drop=FALSE], NULL, along=3)
 				if(dim(tmp)[2] > dim(btf)[2]+1) # if dimension of births doesn't match population
 					tmp <- tmp[,-(1:(dim(tmp)[2]-dim(btf)[2]-1)),, drop=FALSE]
 				asfert <- 2*(btm + btf)/(tmp[,-dim(tmp)[2],,drop=FALSE] + tmp[,-1,,drop=FALSE])
 				tfr <- apply(asfert, c(2,3), sum)
 				pasfert <- asfert/abind(tfr, NULL, along=0)[rep(1,dim(asfert)[1]),,,drop=FALSE]*100
 				rm(tmp, tfr)
-				# TODO: mxm, mxf, mxm.hch, mxf.hch
+				# mortality
+				mxm <- aggrabrdeathsobs[["male"]] / aggrabrpopobs[["male"]] 
+				mxf <- aggrabrdeathsobs[["female"]] / aggrabrpopobs[["female"]]
+				dimnames(mxm)[1:2] <- dimnames(mxf)[1:2] <- list(
+				    c(0,1,seq(5, length = dim(mxm)[1]-2, by = 5)), dimnames(btm)[[2]])
 			})
-			save(btm, btf, deathsm, deathsf, migm, migf, asfert, pasfert,
-				btm.hch, btf.hch, deathsm.hch, deathsf.hch, asfert.hch, pasfert.hch,
+			save(btm, btf, deathsm, deathsf, mxm, mxf, migm, migf, asfert, pasfert,
+				btm.hch, btf.hch, deathsm.hch, deathsf.hch, asfert.hch, pasfert.hch, mxm.hch, mxf.hch,
 				observed, file=file.path(outdir, paste0('vital_events_country', id, '.rda')))
-		}		
+		}
+		#stop("")
 		quant[id.idx,,] = apply(totp, 1, quantile, quantiles.to.keep, na.rm = TRUE)
 		mean_sd[id.idx,1,] <- apply(totp, 1, mean, na.rm = TRUE)
 		mean_sd[id.idx,2,] = apply(totp, 1, sd, na.rm = TRUE)
@@ -369,13 +489,13 @@ pop.aggregate.countries <- function(pop.pred, regions, name, verbose=verbose, ad
 		mean_sdF[id.idx,1,] <- apply(sstotpf, 1, mean, na.rm = TRUE)
 		mean_sdF[id.idx,2,] = apply(sstotpf, 1, sd, na.rm = TRUE)
 		aggregated.countries[[as.character(id)]] <- countries
-		aggr.obs.data[['male']] <- rbind(aggr.obs.data[['male']], aggr.obs.dataM)
-		aggr.obs.data[['female']] <- rbind(aggr.obs.data[['female']], aggr.obs.dataF)
+		for(sex in names(aggrobs)) 
+		    aggr.obs.data[[sex]] <- rbind(aggr.obs.data[[sex]], aggrobs[[sex]])
 	}
 	aggr.pred <- pop.pred
 	which.reg.index <- function(x, set) return(which(set == x))
 	reg.idx <- sapply(regions[valid.regions], which.reg.index, set=UNlocations[,'country_code']) 
-	aggr.pred$countries=data.frame(code=UNlocations[reg.idx, 'country_code'], name=UNlocations[reg.idx, 'name'])
+	aggr.pred$countries <- data.frame(code=UNlocations[reg.idx, 'country_code'], name=UNlocations[reg.idx, 'name'])
 	aggr.pred$quantiles <- quant
 	aggr.pred$quantilesM <- quantM
 	aggr.pred$quantilesF <- quantF
